﻿/*
  Copyright (C) MOXA Inc. All rights reserved.
  This software is distributed under the terms of the
  MOXA License.  See the file COPYING-MOXA for details.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.ServiceProcess;
using System.Threading;
using System.Security.Principal;
using System.Security.AccessControl;
using mxirigcs;

namespace mxIrigbCardConf
{
    public partial class FormMxIrigbConfig : Form
    {
        class PortInfo
        {
            public string name;
            public _PORT_LIST_ port = _PORT_LIST_.PORT_UNKNOWN;
            public bool supportDifferential = true;
            public _PARITY_CHECK_MODE_ parityCheckMode = _PARITY_CHECK_MODE_.PARITY_CHECK_EVEN;

            public _SIGNAL_TYPE_ signalType = _SIGNAL_TYPE_.TYPE_UNKNOWN;
            public _OUTPUT_MODE_ outputMode = _OUTPUT_MODE_.MODE_UNKNOWN;
            public bool inverse = false;
        };

        string[] irigSignalTypeString = new string[] { "TTL", "Differential" };
        string[] irigSignalPolarityString = new string[] { "Normal", "Inverse" };
        string[] irigSignalInputParityString = new string[] { "Even", "Odd", "None" };
        string[] irigSignalOutputParityString = new string[] { "Even", "Odd" };
        string[] irigSignalOutputModeString = new string[] 
            { "From Fiber In", "From Port 1 In", "IRIG-B from internal RTC", "PPS from internal RTC" };
        string[] irigSyncSourceString = new string[] { "Free run", "Fiber In", "Port 1 In" };

        string[] irigSignalStatusString = new string [] 
            { "Normal", "Off Line", "Frame Error", "Parity Error" };

        string[] irigTimeQualityString = new string[] {
            "Normal operation",
            "Accuracy < 1ns",
            "Accuracy < 10ns",
            "Accuracy < 100ns",
            "Accuracy < 1us",
            "Accuracy < 10us",
            "Accuracy < 100us",
            "Accuracy < 1ms",
            "Accuracy < 10ms",
            "Accuracy < 100ms",
            "Accuracy < 1s",
            "Accuracy < 10s",
            "Unknown accuracy, 0xC",
            "Unknown accuracy, 0xD",
            "Unknown accuracy, 0xE",
            "Time not reliable" };


        IntPtr irigbCardHandle;
        BackgroundWorker workerReportStatus;
        BackgroundWorker workerReportDiStatus;
        BackgroundWorker workerSetTimeToRtc;
        LinkedList<PortInfo> inPortList;
        LinkedList<PortInfo> outPortList;
        int hardwareId = (int)_IRIGB_BOARD_HWID_.MAX_BOARD_HWID;

        public FormMxIrigbConfig()
        {
            InitializeComponent();

            Load += new EventHandler(FormMxIrigbConfig_Load);
            FormClosed += new FormClosedEventHandler(FormMxIrigbConfig_FormClosed);
            FormClosing += new FormClosingEventHandler(FormMxIrigbConfig_FormClosing);
        }

        void FormMxIrigbConfig_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (workerReportStatus!=null)
                if (workerReportStatus.IsBusy)
                    workerReportStatus.CancelAsync();

            if (workerReportDiStatus != null)
                if (workerReportDiStatus.IsBusy)
                    workerReportDiStatus.CancelAsync();

            if (workerSetTimeToRtc != null)
                if (workerSetTimeToRtc.IsBusy)
                    workerSetTimeToRtc.CancelAsync();
        }

        void FormMxIrigbConfig_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (workerReportStatus != null)
            {
                workerReportStatus.Dispose();
                workerReportStatus = null;
            }

            if (workerReportDiStatus != null)
            {
                workerReportDiStatus.Dispose();
                workerReportDiStatus = null;
            }

            if (workerSetTimeToRtc != null)
            {
                workerSetTimeToRtc.Dispose();
                workerSetTimeToRtc = null;
            }

            mxIrig.mxIrigbClose(irigbCardHandle);
        }

        void FormMxIrigbConfig_Load(object sender, EventArgs e)
        {
            irigbCardHandle = mxIrig.mxIrigbOpen(0);
            if (irigbCardHandle.ToInt32() == -1 || irigbCardHandle.ToInt32() == 0)
            {
                MessageBox.Show("Moxa Irig-B device not found!");
                Close();
            }
            else
            {
                inPortList = new LinkedList<PortInfo>();
                outPortList = new LinkedList<PortInfo>();
                if (mxIrig.mxIrigbGetHardwareID(irigbCardHandle, ref hardwareId))
                {
                    if (hardwareId == (int)_IRIGB_BOARD_HWID_.DA_IRIGB_S)
                    {
                        Text += " (DA-IRIGB-S)";

                        // inport support list
                        PortInfo PortInfo = new PortInfo();
                        PortInfo.name = "Fiber";
                        PortInfo.port = _PORT_LIST_.PORT_FIBER;
                        PortInfo.supportDifferential = false;
                        inPortList.AddLast(PortInfo);

                        PortInfo = new PortInfo();
                        PortInfo.name = "Port 1";
                        PortInfo.port = _PORT_LIST_.PORT_1;
                        PortInfo.supportDifferential = true;
                        inPortList.AddLast(PortInfo);

                        // outport support list
                        PortInfo = new PortInfo();
                        PortInfo.name = "Port 1";
                        PortInfo.port = _PORT_LIST_.PORT_1;
                        PortInfo.supportDifferential = true;
                        outPortList.AddLast(PortInfo);

                        PortInfo = new PortInfo();
                        PortInfo.name = "Port 2";
                        PortInfo.port = _PORT_LIST_.PORT_2;
                        PortInfo.supportDifferential = true;
                        outPortList.AddLast(PortInfo);

                        PortInfo = new PortInfo();
                        PortInfo.name = "Port 3";
                        PortInfo.port = _PORT_LIST_.PORT_3;
                        PortInfo.supportDifferential = true;
                        outPortList.AddLast(PortInfo);

                        PortInfo = new PortInfo();
                        PortInfo.name = "Port 4";
                        PortInfo.port = _PORT_LIST_.PORT_4;
                        PortInfo.supportDifferential = true;
                        outPortList.AddLast(PortInfo);

                        // DI / DO not support
                        groupBoxDin.Enabled = false;
                        groupBoxDout.Enabled = false;

                        comboBoxTimeSyncSrc.Items.AddRange(irigSyncSourceString);
                    }
                    else if (hardwareId == (int)_IRIGB_BOARD_HWID_.DA_IRIGB_4DIO_PCI104)
                    {
                        Text += " (DA-IRIGB-4DIO-PCI104)";

                        // inport support list
                        PortInfo PortInfo = new PortInfo();
                        PortInfo.name = "Port 1";
                        PortInfo.port = _PORT_LIST_.PORT_1;
                        PortInfo.supportDifferential = true;
                        inPortList.AddLast(PortInfo);

                        // outport support list
                        PortInfo.name = "Port 1";
                        PortInfo.port = _PORT_LIST_.PORT_1;
                        PortInfo.supportDifferential = true;
                        outPortList.AddLast(PortInfo);

                        groupBoxOutput.Enabled = false;
                        groupBoxInternalRtcOutput.Enabled = false;
                        textBoxStatusPortFiber.Visible = false;
                        labelStatusIrig0.Visible = false;
                        //checkBoxDout3.Visible = false;
                        checkBoxDin3.Visible = false;

                        comboBoxTimeSyncSrc.Items.Add(irigSyncSourceString[(int)_RTC_SYNC_SOURCE_.TIMESRC_FREERUN]);
                        comboBoxTimeSyncSrc.Items.Add(irigSyncSourceString[(int)_RTC_SYNC_SOURCE_.TIMESRC_PORT1]);
                    }
                    else
                    {
                        MessageBox.Show("Moxa Irig-B device not found!");
                        Close();
                        return;
                    }
                }

                // 
                comboBoxInputSignalType.Items.AddRange(irigSignalTypeString);
                comboBoxInputPolarity.Items.AddRange(irigSignalPolarityString);
                comboBoxInputParityMode.Items.AddRange(irigSignalInputParityString);
                comboBoxOutputSignalType.Items.AddRange(irigSignalTypeString);
                comboBoxOutputPolarity.Items.AddRange(irigSignalPolarityString);
                comboBoxOutputMode.Items.AddRange(irigSignalOutputModeString);
                comboBoxOutputParityMode.Items.AddRange(irigSignalOutputParityString);

                GetParameter();

                // Input Config
                foreach (PortInfo port in inPortList)
                {
                    mxIrig.mxIrigbGetInputSignalType(this.irigbCardHandle,
                        port.port, ref port.signalType, ref port.inverse);
                    if (port.signalType == _SIGNAL_TYPE_.TYPE_UNKNOWN)
                        port.signalType = _SIGNAL_TYPE_.TYPE_TTL;
                    comboBoxInputPort.Items.Add(port.name);
                }

                comboBoxInputSignalType.SelectedIndexChanged += new EventHandler(comboBoxInputSignalType_SelectedIndexChanged);
                comboBoxInputPolarity.SelectedIndexChanged += new EventHandler(comboBoxInputPolarity_SelectedIndexChanged);
                comboBoxInputParityMode.SelectedIndexChanged += new EventHandler(comboBoxInputParityMode_SelectedIndexChanged);
                comboBoxInputPort.SelectedIndexChanged += new EventHandler(comboBoxInputPort_SelectedIndexChanged);
                comboBoxInputPort.SelectedIndex = 0;

                // Output Config
                foreach (PortInfo port in outPortList)
                {
                    mxIrig.mxIrigbGetOutputSignalType(this.irigbCardHandle,
                        port.port, ref port.signalType, ref port.outputMode, ref port.inverse);
                    if (port.signalType == _SIGNAL_TYPE_.TYPE_UNKNOWN)
                        port.signalType = _SIGNAL_TYPE_.TYPE_TTL;
                    if (port.outputMode == _OUTPUT_MODE_.MODE_UNKNOWN)
                        port.outputMode = _OUTPUT_MODE_.MODE_FROM_PORT1_IN;
                    comboBoxOutputPort.Items.Add(port.name);
                }

                comboBoxOutputSignalType.SelectedIndexChanged += new EventHandler(comboBoxOutputSignalType_SelectedIndexChanged);
                comboBoxOutputPolarity.SelectedIndexChanged += new EventHandler(comboBoxOutputPolarity_SelectedIndexChanged);
                comboBoxOutputMode.SelectedIndexChanged += new EventHandler(comboBoxOutputMode_SelectedIndexChanged);
                comboBoxOutputPort.SelectedIndexChanged += new EventHandler(comboBoxOutputPort_SelectedIndexChanged);
                comboBoxOutputPort.SelectedIndex = 0;

                _PARITY_CHECK_MODE_ outParityMode = _PARITY_CHECK_MODE_.PARITY_CHECK_EVEN;
                mxIrig.mxIrigbGetOutputParityCheckMode(this.irigbCardHandle, ref outParityMode);
                comboBoxOutputParityMode.SelectedIndex = (int)outParityMode;

                // PPS width
                int ppsWidth = 0;
                mxIrig.mxIrigbGetPpsWidth(irigbCardHandle, ref ppsWidth);
                textBoxPpsWidth.Text = ppsWidth.ToString();

                // Time Source
                _RTC_SYNC_SOURCE_ rtcSource = _RTC_SYNC_SOURCE_.TIMESRC_FREERUN;
                mxIrig.mxIrigbGetSyncTimeSrc(irigbCardHandle, ref rtcSource);
                if (hardwareId == (int) _IRIGB_BOARD_HWID_.DA_IRIGB_4DIO_PCI104)
                {
                    switch (rtcSource)
                    {
                        case _RTC_SYNC_SOURCE_.TIMESRC_PORT1:
                            comboBoxTimeSyncSrc.SelectedIndex = 1;
                            break;
                        default:
                            comboBoxTimeSyncSrc.SelectedIndex = 0;
                            break;
                    }
                }
                else
                    comboBoxTimeSyncSrc.SelectedIndex = (int)rtcSource;

                // Sync Time to RTC
                workerSetTimeToRtc = new BackgroundWorker();
                workerSetTimeToRtc.WorkerSupportsCancellation = true;
                workerSetTimeToRtc.DoWork += new DoWorkEventHandler(workerSetTimeToRtc_DoWork);
                workerSetTimeToRtc.RunWorkerCompleted += new RunWorkerCompletedEventHandler(workerSetTimeToRtc_RunWorkerCompleted);

                // Monitor Status
                workerReportStatus = new BackgroundWorker();
                workerReportStatus.WorkerSupportsCancellation = true;
                workerReportStatus.DoWork += new DoWorkEventHandler(workerReportStatus_DoWork);
                workerReportStatus.RunWorkerAsync();

                // check box for Din
                if (groupBoxDin.Enabled)
                {
                    workerReportDiStatus = new BackgroundWorker();
                    workerReportDiStatus.WorkerSupportsCancellation = true;
                    workerReportDiStatus.DoWork += new DoWorkEventHandler(workerReportDiStatus_DoWork);
                    workerReportDiStatus.RunWorkerAsync();
                }

                // check box for Dout
                if (groupBoxDout.Enabled)
                {
                    int value = 0;
                    mxirigcs.mxIrig.mxIrigbGetDigitalOutputSignal(irigbCardHandle, 0, ref value);
                    checkBoxDout0.Checked = value > 0;
                    mxirigcs.mxIrig.mxIrigbGetDigitalOutputSignal(irigbCardHandle, 1, ref value);
                    checkBoxDout1.Checked = value > 0;
                    mxirigcs.mxIrig.mxIrigbGetDigitalOutputSignal(irigbCardHandle, 2, ref value);
                    checkBoxDout2.Checked = value > 0;
                    mxirigcs.mxIrig.mxIrigbGetDigitalOutputSignal(irigbCardHandle, 3, ref value);
                    checkBoxDout3.Checked = value > 0;

                    checkBoxDout0.CheckedChanged += new EventHandler(checkBoxDout_CheckedChanged);
                    checkBoxDout1.CheckedChanged += new EventHandler(checkBoxDout_CheckedChanged);
                    checkBoxDout2.CheckedChanged += new EventHandler(checkBoxDout_CheckedChanged);
                    checkBoxDout3.CheckedChanged += new EventHandler(checkBoxDout_CheckedChanged);
                }

                // buttons
                buttonOk.Click += new EventHandler(buttonOk_Click);
                buttonCancel.Click += new EventHandler(buttonCancel_Click);
                buttonSetTimeToRtc.Click += new EventHandler(buttonSetTimeToRtc_Click);
            }
        }

        void checkBoxDout_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkbox = sender as CheckBox;
            int value;
            int port = -1;

            if (checkbox.Name == checkBoxDout0.Name)
                port = 0;
            else if (checkbox.Name == checkBoxDout1.Name)
                port = 1;
            else if (checkbox.Name == checkBoxDout2.Name)
                port = 2;
            else if (checkbox.Name == checkBoxDout3.Name)
                port = 3;
            else
                return;

            if (checkbox.Checked)
                value = 1;
            else
                value = 0;

            mxIrig.mxIrigbSetDigitalOutputSignal(this.irigbCardHandle, port, value);
        }

        void buttonCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        void buttonOk_Click(object sender, EventArgs e)
        {
            try
            {
                int ppsWidth = Convert.ToInt32(textBoxPpsWidth.Text);
                if (ppsWidth < 0)
                {
                    MessageBox.Show("Invalid Value!");
                    return;
                }
            }
            catch
            {
                MessageBox.Show("Invalid Value!");
                return;
            }

            try
            {
                int syncPerSec = Convert.ToInt32(textBoxSyncPerSec.Text);
                if (syncPerSec <= 0)
                {
                    MessageBox.Show("Invalid Value!");
                    return;
                }
            }
            catch
            {
                MessageBox.Show("Invalid Value!");
                return;
            }

            SetParameter();

            //
            // Notify Sync. Time service to set new setting without set time to RTC
            //
            try
            {
                EventWaitHandle ewh = null;
                string ewhName = @"Global\mxSyncTime";
                ewh = EventWaitHandle.OpenExisting(ewhName,
                    EventWaitHandleRights.ReadPermissions |
                    EventWaitHandleRights.ChangePermissions);

                // Get the current ACL. This requires  
                // EventWaitHandleRights.ReadPermissions.
                EventWaitHandleSecurity ewhSec = ewh.GetAccessControl();

                string user = Environment.UserDomainName + "\\"
                    + Environment.UserName;

                // First, the rule that denied the current user  
                // the right to enter and release the event must 
                // be removed.
                EventWaitHandleAccessRule rule =
                    new EventWaitHandleAccessRule(user,
                        EventWaitHandleRights.Synchronize |
                        EventWaitHandleRights.Modify,
                        AccessControlType.Deny);
                ewhSec.RemoveAccessRule(rule);

                // Now grant the user the correct rights. 
                // 
                rule = new EventWaitHandleAccessRule(user,
                    EventWaitHandleRights.Synchronize |
                    EventWaitHandleRights.Modify,
                    AccessControlType.Allow);
                ewhSec.AddAccessRule(rule);

                // Update the ACL. This requires 
                // EventWaitHandleRights.ChangePermissions.
                ewh.SetAccessControl(ewhSec);

                // Open the event with (EventWaitHandleRights.Synchronize  
                // | EventWaitHandleRights.Modify), the rights required 
                // to wait on and signal the event. 
                //
                ewh = EventWaitHandle.OpenExisting(ewhName);

                ewh.Set();
            }
            catch
            {
                MessageBox.Show("mxIrigSyncTime Service is not running!");
            }
        }

        void buttonSetTimeToRtc_Click(object sender, EventArgs e)
        {
            buttonSetTimeToRtc.Enabled = false;
            if (!workerSetTimeToRtc.IsBusy)
                workerSetTimeToRtc.RunWorkerAsync();
        }

        void comboBoxOutputPort_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox combobox = sender as ComboBox;
            PortInfo port = outPortList.ElementAt(combobox.SelectedIndex);

            comboBoxOutputSignalType.SelectedIndex = (int)port.signalType;
            comboBoxOutputPolarity.SelectedIndex = port.inverse ? 1 : 0;
            comboBoxOutputMode.SelectedIndex = (int)port.outputMode;
        }

        void comboBoxOutputMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox combobox = sender as ComboBox;
            PortInfo port = outPortList.ElementAt(comboBoxOutputPort.SelectedIndex);

            port.outputMode = (_OUTPUT_MODE_)combobox.SelectedIndex;
        }

        void comboBoxOutputPolarity_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox combobox = sender as ComboBox;
            PortInfo port = outPortList.ElementAt(comboBoxOutputPort.SelectedIndex);

            port.inverse = combobox.SelectedIndex != 0;
        }

        void comboBoxOutputSignalType_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox combobox = sender as ComboBox;
            PortInfo port = outPortList.ElementAt(comboBoxOutputPort.SelectedIndex);

            port.signalType = (_SIGNAL_TYPE_)combobox.SelectedIndex;
        }

        void comboBoxInputPort_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox combobox = sender as ComboBox;
            PortInfo port = inPortList.ElementAt(combobox.SelectedIndex);

            comboBoxInputSignalType.SelectedIndex = (int)port.signalType;
            comboBoxInputPolarity.SelectedIndex = port.inverse ? 1 : 0;
            comboBoxInputParityMode.SelectedIndex = (int)port.parityCheckMode;
        }

        void comboBoxInputSignalType_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox combobox = sender as ComboBox;
            PortInfo port = inPortList.ElementAt(comboBoxInputPort.SelectedIndex);

            if (combobox.SelectedIndex==(int)_SIGNAL_TYPE_.TYPE_DIFFERENTIAL)
            {
                if (!port.supportDifferential)
                {
                    combobox.SelectedIndex = (int)_SIGNAL_TYPE_.TYPE_TTL;
                }
            }

            port.signalType = (_SIGNAL_TYPE_)combobox.SelectedIndex;
        }

        void comboBoxInputParityMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox combobox = sender as ComboBox;
            PortInfo port = inPortList.ElementAt(comboBoxInputPort.SelectedIndex);
            port.parityCheckMode = (_PARITY_CHECK_MODE_)combobox.SelectedIndex;
        }

        void comboBoxInputPolarity_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox combobox = sender as ComboBox;
            PortInfo port = inPortList.ElementAt(comboBoxInputPort.SelectedIndex);
            port.inverse = combobox.SelectedIndex != 0;
        }

        void workerReportStatus_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker bw = sender as BackgroundWorker;
            RTCTIME rtctime = new RTCTIME();
            int waitms = 0;

            while (true)
            {
                if (bw.CancellationPending)
                {
                    break;
                }

                //
                // Show IRIG signal status
                _IRIG_SIGNAL_STATUS_ signalStatus = _IRIG_SIGNAL_STATUS_.IRIG_STATUS_UNKNOWN;

                if (hardwareId != (int) _IRIGB_BOARD_HWID_.DA_IRIGB_4DIO_PCI104)
                {
                    if (mxIrig.mxIrigbGetSignalStatus(irigbCardHandle,
                        _RTC_SYNC_SOURCE_.TIMESRC_FIBER, ref signalStatus))
                    {
                        if (textBoxStatusPortFiber.InvokeRequired)
                        {
                            textBoxStatusPortFiber.Invoke(new MethodInvoker(delegate
                            {
                                textBoxStatusPortFiber.Text = irigSignalStatusString[(int)signalStatus];
                            }));
                        }
                        else
                        {
                            textBoxStatusPortFiber.Text = irigSignalStatusString[(int)signalStatus];
                        }
                    }
                }

                if (mxIrig.mxIrigbGetSignalStatus(irigbCardHandle, 
                    _RTC_SYNC_SOURCE_.TIMESRC_PORT1, ref signalStatus))
                {
                    if (textBoxStatusPort1.InvokeRequired)
                    {
                        textBoxStatusPort1.Invoke(new MethodInvoker(delegate
                        {
                            textBoxStatusPort1.Text = irigSignalStatusString[(int)signalStatus];
                        }));
                    }
                    else
                    {
                        textBoxStatusPort1.Text = irigSignalStatusString[(int)signalStatus];
                    }
                }

                //
                // Show RTC time
                if (mxIrig.mxIrigbGetTime( irigbCardHandle, out rtctime))
                {
                    string timestr = 
                        string.Format("{0:0000}/{1:00}/{2:00} ", rtctime.year, rtctime.mon, rtctime.mday) +
                        string.Format("{0:00}:{1:00}:{2:00}", rtctime.hour, rtctime.min, rtctime.sec)
                        ;
                    waitms = 1000 - (rtctime.nanosec/1000000);
                    if (textBoxStatusTime.InvokeRequired)
                    {
                        textBoxStatusTime.Invoke(new MethodInvoker(delegate
                        {
                            textBoxStatusTime.Text = timestr;
                        }));
                    }
                    else
                    {
                        textBoxStatusTime.Text = timestr;
                    }

                    // Time Zone
                    string timezonestr = "";
                    if (rtctime.tzs > 0)
                        timezonestr += "-";
                    else
                        timezonestr += "+";
                    timezonestr += rtctime.tz.ToString();
                    if (rtctime.tzh > 0)
                        timezonestr += ".5";

                    if (textBoxStatusTimeZone.InvokeRequired)
                    {
                        textBoxStatusTimeZone.Invoke(new MethodInvoker(delegate
                        {
                            textBoxStatusTimeZone.Text = timezonestr;
                        }));
                    }
                    else
                    {
                        textBoxStatusTimeZone.Text = timezonestr;
                    }

                    // Time Quality
                    if (textBoxStatusTimeQuality.InvokeRequired)
                    {
                        textBoxStatusTimeQuality.Invoke(new MethodInvoker(delegate
                        {
                            textBoxStatusTimeQuality.Text = irigTimeQualityString[rtctime.tq];
                        }));
                    }
                    else
                    {
                        textBoxStatusTimeQuality.Text = irigTimeQualityString[rtctime.tq];
                    }

                    // Leap Second
                    if (radioButtonStatusLsp.InvokeRequired)
                    {
                        radioButtonStatusLsp.Invoke(new MethodInvoker(delegate
                        {
                            radioButtonStatusLsp.Checked = rtctime.lsp > 0;
                        }));
                    }
                    else
                    {
                        radioButtonStatusLsp.Checked = rtctime.lsp > 0;
                    }
                    if (radioButtonStatusLs.InvokeRequired)
                    {
                        radioButtonStatusLs.Invoke(new MethodInvoker(delegate
                        {
                            radioButtonStatusLs.Checked = rtctime.ls > 0;
                        }));
                    }
                    else
                    {
                        radioButtonStatusLs.Checked = rtctime.ls > 0;
                    }

                    // Daynight Saving
                    if (radioButtonStatusDsp.InvokeRequired)
                    {
                        radioButtonStatusDsp.Invoke(new MethodInvoker(delegate
                        {
                            radioButtonStatusDsp.Checked = rtctime.dsp > 0;
                        }));
                    }
                    else
                    {
                        radioButtonStatusDsp.Checked = rtctime.dsp > 0;
                    }
                    if (radioButtonStatusDst.InvokeRequired)
                    {
                        radioButtonStatusDst.Invoke(new MethodInvoker(delegate
                        {
                            radioButtonStatusDst.Checked = rtctime.dst > 0;
                        }));
                    }
                    else
                    {
                        radioButtonStatusDst.Checked = rtctime.dst > 0;
                    }
                }

                // Next read time
                System.Threading.Thread.Sleep(waitms);
            }
        }

        void workerReportDiStatus_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker bw = sender as BackgroundWorker;
            const int waitms = 200;

            while (true)
            {
                if (bw.CancellationPending)
                {
                    break;
                }

                int value = 0;
                if (mxIrig.mxIrigbGetDigitalInputSignal(irigbCardHandle, 0, ref value))
                {
                    if (checkBoxDin0.InvokeRequired)
                    {
                        checkBoxDin0.Invoke(new MethodInvoker(delegate {
                            checkBoxDin0.Checked = value > 0;
                        }));
                    }
                    else
                    {
                        checkBoxDin0.Checked = value > 0;
                    }
                }

                if (mxIrig.mxIrigbGetDigitalInputSignal(irigbCardHandle, 1, ref value))
                {
                    if (checkBoxDin1.InvokeRequired)
                    {
                        checkBoxDin1.Invoke(new MethodInvoker(delegate
                        {
                            checkBoxDin1.Checked = value > 0;
                        }));
                    }
                    else
                    {
                        checkBoxDin1.Checked = value > 0;
                    }
                }

                if (mxIrig.mxIrigbGetDigitalInputSignal(irigbCardHandle, 2, ref value))
                {
                    if (checkBoxDin2.InvokeRequired)
                    {
                        checkBoxDin2.Invoke(new MethodInvoker(delegate
                        {
                            checkBoxDin2.Checked = value > 0;
                        }));
                    }
                    else
                    {
                        checkBoxDin2.Checked = value > 0;
                    }
                }

                if (mxIrig.mxIrigbGetDigitalInputSignal(irigbCardHandle, 3, ref value))
                {
                    if (checkBoxDin3.InvokeRequired)
                    {
                        checkBoxDin3.Invoke(new MethodInvoker(delegate
                        {
                            checkBoxDin3.Checked = value > 0;
                        }));
                    }
                    else
                    {
                        checkBoxDin3.Checked = value > 0;
                    }
                }

                // Next read time
                System.Threading.Thread.Sleep(waitms);
            }


        }

        void workerSetTimeToRtc_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            buttonSetTimeToRtc.Enabled = true;
        }

        void workerSetTimeToRtc_DoWork(object sender, DoWorkEventArgs e)
        {
            mxIrig.mxIrigbSyncTime(irigbCardHandle, true); // Set Local Time to IRIG RTC
        }

        private void GetParameter()
        {
            string keyPathString = "SOFTWARE\\MOXA\\IRIGPCI";
            Microsoft.Win32.RegistryKey start = Microsoft.Win32.Registry.LocalMachine;
            Microsoft.Win32.RegistryKey programName = start.OpenSubKey(keyPathString);

            if (programName == null)
                return;

            int signalType = 0;
            int inverse = 0;
            int outputMode = 0;
            int parityCheckMode = 0;

            //
            // config input interface
            try
            {
                signalType = (int)programName.GetValue("InPortFiberSignalType", 0);
                inverse = (int)programName.GetValue("InPortFiberInverse", 0);
                parityCheckMode = (int)programName.GetValue("InPortFiberParity", 0);
            }
            catch
            {
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 0;
                parityCheckMode = (int)_PARITY_CHECK_MODE_.PARITY_CHECK_EVEN;
            }
            mxIrig.mxIrigbSetInputSignalType(irigbCardHandle, _PORT_LIST_.PORT_FIBER,
                (_SIGNAL_TYPE_)signalType, inverse != 0);
            mxIrig.mxIrigbSetInputParityCheckMode(irigbCardHandle,
                _RTC_SYNC_SOURCE_.TIMESRC_FIBER, (_PARITY_CHECK_MODE_)parityCheckMode);

            try
            {
                signalType = (int)programName.GetValue("InPort1SignalType", 0);
                inverse = (int)programName.GetValue("InPort1Inverse", 0);
                parityCheckMode = (int)programName.GetValue("InPort1Parity", 0);
            }
            catch
            {
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 0;
                parityCheckMode = (int)_PARITY_CHECK_MODE_.PARITY_CHECK_EVEN;
            }
            mxIrig.mxIrigbSetInputSignalType(irigbCardHandle, _PORT_LIST_.PORT_1,
                (_SIGNAL_TYPE_)signalType, inverse != 0);
            mxIrig.mxIrigbSetInputParityCheckMode(irigbCardHandle,
                _RTC_SYNC_SOURCE_.TIMESRC_PORT1, (_PARITY_CHECK_MODE_)parityCheckMode);

            //
            // config output interface
            try
            {
                outputMode = (int)programName.GetValue("OutPort1OutputMode", 0);
                signalType = (int)programName.GetValue("OutPort1SignalType", 0);
                inverse = (int)programName.GetValue("OutPort1Inverse", 0);
            }
            catch
            {
                outputMode = (int)_OUTPUT_MODE_.MODE_FROM_FIBER_IN;
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 1;
            }
            mxIrig.mxIrigbSetOutputSignalType(irigbCardHandle, _PORT_LIST_.PORT_1,
                (_SIGNAL_TYPE_)signalType, (_OUTPUT_MODE_)outputMode, inverse != 0);

            try
            {
                outputMode = (int)programName.GetValue("OutPort2OutputMode", 0);
                signalType = (int)programName.GetValue("OutPort2SignalType", 0);
                inverse = (int)programName.GetValue("OutPort2Inverse", 0);
            }
            catch
            {
                outputMode = (int)_OUTPUT_MODE_.MODE_FROM_FIBER_IN;
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 1;
            }
            mxIrig.mxIrigbSetOutputSignalType(irigbCardHandle, _PORT_LIST_.PORT_2,
                (_SIGNAL_TYPE_)signalType, (_OUTPUT_MODE_)outputMode, inverse != 0);

            try
            {
                outputMode = (int)programName.GetValue("OutPort3OutputMode", 0);
                signalType = (int)programName.GetValue("OutPort3SignalType", 0);
                inverse = (int)programName.GetValue("OutPort3Inverse", 0);
            }
            catch
            {
                outputMode = (int)_OUTPUT_MODE_.MODE_FROM_FIBER_IN;
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 1;
            }
            mxIrig.mxIrigbSetOutputSignalType(irigbCardHandle, _PORT_LIST_.PORT_3,
                (_SIGNAL_TYPE_)signalType, (_OUTPUT_MODE_)outputMode, inverse != 0);

            try
            {
                outputMode = (int)programName.GetValue("OutPort4OutputMode", 0);
                signalType = (int)programName.GetValue("OutPort4SignalType", 0);
                inverse = (int)programName.GetValue("OutPort4Inverse", 0);
            }
            catch
            {
                outputMode = (int)_OUTPUT_MODE_.MODE_FROM_FIBER_IN;
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 1;
            }
            mxIrig.mxIrigbSetOutputSignalType(irigbCardHandle, _PORT_LIST_.PORT_4,
                (_SIGNAL_TYPE_)signalType, (_OUTPUT_MODE_)outputMode, inverse != 0);

            int ppsWidth = 0;
            try
            {
                ppsWidth = (int)programName.GetValue("PpsOutputWidth", 0);
            }
            catch { }
            mxIrig.mxIrigbSetPpsWidth(irigbCardHandle, ppsWidth);

            int outputParity = 0;
            try
            {
                outputParity = (int)programName.GetValue("IrigBOutputParity", 0);
            }
            catch { }
            mxIrig.mxIrigbSetOutputParityCheckMode(irigbCardHandle, (_PARITY_CHECK_MODE_)outputParity);

            //
            // Sync Time Source
            int timeSource = 0;
            try
            {
                timeSource = (int)programName.GetValue("SyncTimeSource", 0);
            }
            catch { }
            mxIrig.mxIrigbSetSyncTimeSrc(irigbCardHandle, (_RTC_SYNC_SOURCE_)timeSource);

            int syncTime = 0;
            int syncTimeEnable = 0;
            try
            {
                syncTime = (int)programName.GetValue("SyncTime", 0);
                syncTimeEnable = (int)programName.GetValue("SyncTimeEnable", 0);
            }
            catch { }
            textBoxSyncPerSec.Text = syncTime.ToString();
            checkBoxSyncPerTime.Checked = syncTimeEnable > 0;

            programName.Close();
            programName = null;
        }

        private void SetParameter()
        {
            string keyPathString = "SOFTWARE\\MOXA\\IRIGPCI";
            Microsoft.Win32.RegistryKey start = Microsoft.Win32.Registry.LocalMachine;
            Microsoft.Win32.RegistryKey programName = start.OpenSubKey(keyPathString, true);
            if (programName == null)
            {
                programName = start.CreateSubKey(keyPathString);
            }

            if (programName == null)
                return;

            int inverse = 0;

            //
            // set input interface
            try
            {
                foreach (PortInfo port in inPortList)
                {
                    string keyname = "";
                    if (port.port == _PORT_LIST_.PORT_FIBER)
                        keyname = "Fiber";
                    else if (port.port == _PORT_LIST_.PORT_1)
                        keyname = "1";
                    else
                        continue;

                    programName.SetValue(string.Format("InPort{0}SignalType", keyname),
                        port.signalType, Microsoft.Win32.RegistryValueKind.DWord);
                    if (port.inverse)
                        inverse = 1;
                    else
                        inverse = 0;
                    programName.SetValue(string.Format("InPort{0}Inverse", keyname),
                        inverse, Microsoft.Win32.RegistryValueKind.DWord);
                    programName.SetValue(string.Format("InPort{0}Parity", keyname),
                        port.parityCheckMode, Microsoft.Win32.RegistryValueKind.DWord);
                }
            }
            catch { }

            //
            // set output interface
            try
            {
                foreach (PortInfo port in outPortList)
                {
                    string keyname = "";

                    if (port.port == _PORT_LIST_.PORT_1)
                        keyname = "1";
                    else if (port.port == _PORT_LIST_.PORT_2)
                        keyname = "2";
                    else if (port.port == _PORT_LIST_.PORT_3)
                        keyname = "3";
                    else if (port.port == _PORT_LIST_.PORT_4)
                        keyname = "4";
                    else
                        continue;

                    programName.SetValue(string.Format("OutPort{0}OutputMode", keyname), 
                        port.outputMode, Microsoft.Win32.RegistryValueKind.DWord);
                    programName.SetValue(string.Format("OutPort{0}SignalType", keyname),
                        port.signalType, Microsoft.Win32.RegistryValueKind.DWord);
                    if (port.inverse)
                        inverse = 1;
                    else
                        inverse = 0;
                    programName.SetValue(string.Format("OutPort{0}Inverse", keyname),
                        inverse, Microsoft.Win32.RegistryValueKind.DWord);
                }
            }
            catch { }

            try
            {
                int ppsWidth = 0;
                ppsWidth = Convert.ToInt32(textBoxPpsWidth.Text);
                programName.SetValue("PpsOutputWidth", ppsWidth, Microsoft.Win32.RegistryValueKind.DWord);
            }
            catch { }

            try
            {
                int outputParity = 0;
                outputParity = comboBoxOutputParityMode.SelectedIndex;
                programName.SetValue("IrigBOutputParity", outputParity, Microsoft.Win32.RegistryValueKind.DWord);
            }
            catch { }

            //
            // Sync Time Source
            try
            {
                int syncTime = 0;
                int syncTimeEnable = 0;
                
                _RTC_SYNC_SOURCE_ syncSource;
                if (hardwareId == (int) _IRIGB_BOARD_HWID_.DA_IRIGB_4DIO_PCI104)
                {
                    switch (comboBoxTimeSyncSrc.SelectedIndex)
                    {
                        case 1:
                            syncSource = _RTC_SYNC_SOURCE_.TIMESRC_PORT1;
                            break;
                        default:
                            syncSource = _RTC_SYNC_SOURCE_.TIMESRC_FREERUN;
                            break;
                    }
                }
                else
                {
                    syncSource = (_RTC_SYNC_SOURCE_)comboBoxTimeSyncSrc.SelectedIndex;
                }
                
                programName.SetValue("SyncTimeSource", (int)syncSource, Microsoft.Win32.RegistryValueKind.DWord);
                syncTime = Convert.ToInt32(textBoxSyncPerSec.Text);
                programName.SetValue("SyncTime", syncTime, Microsoft.Win32.RegistryValueKind.DWord);
                if (checkBoxSyncPerTime.Checked)
                    syncTimeEnable = 1;
                else
                    syncTimeEnable = 0;
                programName.SetValue("SyncTimeEnable", syncTimeEnable, Microsoft.Win32.RegistryValueKind.DWord);
            }
            catch { }

            programName.Close();
            programName = null;
        }

    }
}
