﻿/*
  Copyright (C) MOXA Inc. All rights reserved.
  This software is distributed under the terms of the
  MOXA License.  See the file COPYING-MOXA for details.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceProcess;
using System.Text;
using System.Configuration.Install;
using System.Collections;

#if DEBUG
using System.Runtime.InteropServices;
#endif

namespace mxSyncTimeSvc
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        static void Main(string[] args)
        {
            // Set work directory to program directory
            System.IO.Directory.SetCurrentDirectory(System.AppDomain.CurrentDomain.BaseDirectory);

            bool runAsConsole = false;

#if DEBUG
            runAsConsole = true;
#endif

#if DEBUG
            AllocConsole();
#endif

            if (args.Length > 0)
            {
                if (args[0].ToLower() == "-c")
                {
                    runAsConsole = true;
                }
                if (args[0].ToLower() == "-i" || args[0].ToLower() == "/i")
                {
                    Install(false, args);
                    return;
                }
                if (args[0].ToLower() == "-u" || args[0].ToLower() == "/u")
                {
                    Install(true, args);
                    return;
                }
            }

            if (runAsConsole)
            {
                ServiceSyncTime svc = new ServiceSyncTime();
                svc.StartService(args);
                Console.WriteLine("Press Q to quit");
                while (Char.ToUpper(Console.ReadKey().KeyChar) != 'Q') ;
                svc.StopService();
                return;
            }
            else
            {
                ServiceBase[] ServicesToRun;
                ServicesToRun = new ServiceBase[] 
			    { 
				new ServiceSyncTime() 
			    };
                ServiceBase.Run(ServicesToRun);
            }

#if DEBUG
            Console.ReadLine();
            FreeConsole();
#endif

        }

        static void Install(bool undo, string[] args)
        {
            try
            {
                string[] cmdline;
                if (undo)
                {
                    cmdline = new string[] { "/u", "/LogFile=" };
                }
                else
                {
                    cmdline = new string[] { "/LogFile=" };
                }

                Console.WriteLine(undo ? "uninstalling" : "installing");
                using (AssemblyInstaller inst = new AssemblyInstaller(typeof(Program).Assembly, cmdline))
                {
                    IDictionary state = new Hashtable();
                    inst.UseNewContext = true;
                    try
                    {
                        if (undo)
                        {
                            inst.Uninstall(state);
                        }
                        else
                        {
                            inst.Install(state);
                            inst.Commit(state);
                        }
                    }
                    catch
                    {
                        try
                        {
                            inst.Rollback(state);
                        }
                        catch { }
                        throw;
                    }
                }
            }
            catch (Exception ex)
            {
                Console.Error.WriteLine(ex.Message);
            }
        }

#if DEBUG
        [DllImport("kernel32.dll")]
        public static extern Boolean AllocConsole();
        [DllImport("kernel32.dll")]
        public static extern Boolean FreeConsole();
#endif

    }
}
