﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using mxirigcs;

namespace mxSyncTimeSvc
{
    public partial class ServiceSyncTime : ServiceBase
    {
        IntPtr irigbCardHandle;
        Timer timerSyncPeriod;
        WaitHandle waitHandle;

        public ServiceSyncTime()
        {
            InitializeComponent();
        }

        public void StartService(string[] args)
        {
            OnStart(args);
        }

        public void StopService()
        {
            OnStop();
        }

        protected override void OnStart(string[] args)
        {
#if DEBUG
            Console.WriteLine("+++Services start");
#endif
            irigbCardHandle = mxIrig.mxIrigbOpen(0);
            if (irigbCardHandle.ToInt32() == -1 || irigbCardHandle.ToInt32() == 0)
            {
                return;
            }

            mxIrig.mxIrigbSyncTime(irigbCardHandle, true); // Set Local Time to IRIG RTC

            Init();

            bool createdNew;
            waitHandle = new EventWaitHandle(false,
                EventResetMode.AutoReset, @"Global\mxSyncTime", out createdNew);
            // createdNew should be 'false' because event already exists
            ThreadPool.RegisterWaitForSingleObject(waitHandle, ReInitSyncTime, null,
                -1, false);

#if DEBUG
            Console.WriteLine("---Services start");
#endif
        }

        protected override void OnStop()
        {
#if DEBUG
            Console.WriteLine("+++Services stop");
#endif

            mxIrig.mxIrigbClose(irigbCardHandle);

#if DEBUG
            Console.WriteLine("---Services stop");
#endif
        }

        private void Init()
        {
            string keyPathString = "SOFTWARE\\MOXA\\IRIGPCI";
            Microsoft.Win32.RegistryKey start = Microsoft.Win32.Registry.LocalMachine;
            Microsoft.Win32.RegistryKey programName = start.OpenSubKey(keyPathString);

            if (programName == null)
                return;

            int signalType = 0;
            int inverse = 0;
            int outputMode = 0;
            int parityCheckMode = 0;

            //
            // config input interface
            try
            {
                signalType = (int)programName.GetValue("InPortFiberSignalType", 0);
                inverse = (int)programName.GetValue("InPortFiberInverse", 0);
                parityCheckMode = (int)programName.GetValue("InPortFiberParity", 0);
            }
            catch
            {
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 0;
                parityCheckMode = (int)_PARITY_CHECK_MODE_.PARITY_CHECK_EVEN;
            }
            Console.WriteLine("InPortFiberSignalType={0}", signalType);
            Console.WriteLine("InPortFiberInverse={0}", inverse);
            Console.WriteLine("InPortFiberParity={0}", parityCheckMode);
            mxIrig.mxIrigbSetInputSignalType(irigbCardHandle, _PORT_LIST_.PORT_FIBER,
                (_SIGNAL_TYPE_)signalType, inverse != 0);
            mxIrig.mxIrigbSetInputParityCheckMode(irigbCardHandle,
                _RTC_SYNC_SOURCE_.TIMESRC_FIBER, (_PARITY_CHECK_MODE_)parityCheckMode);

            try
            {
                signalType = (int)programName.GetValue("InPort1SignalType", 0);
                inverse = (int)programName.GetValue("InPort1Inverse", 0);
                parityCheckMode = (int)programName.GetValue("InPort1Parity", 0);
            }
            catch
            {
                Console.WriteLine("InPort1Signal faile");
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 0;
                parityCheckMode = (int)_PARITY_CHECK_MODE_.PARITY_CHECK_EVEN;
            }
            Console.WriteLine("InPort1SignalType={0}", signalType);
            Console.WriteLine("InPort1Inverse={0}", inverse);
            Console.WriteLine("InPort1Parity={0}", parityCheckMode);
            mxIrig.mxIrigbSetInputSignalType(irigbCardHandle, _PORT_LIST_.PORT_1,
                (_SIGNAL_TYPE_)signalType, inverse != 0);
            mxIrig.mxIrigbSetInputParityCheckMode(irigbCardHandle,
                _RTC_SYNC_SOURCE_.TIMESRC_PORT1, (_PARITY_CHECK_MODE_)parityCheckMode);

            //
            // config output interface
            try
            {
                outputMode = (int)programName.GetValue("OutPort1OutputMode", 0);
                signalType = (int)programName.GetValue("OutPort1SignalType", 0);
                inverse = (int)programName.GetValue("OutPort1Inverse", 0);
            }
            catch
            {
                Console.WriteLine("OutPort1Output faile");
                outputMode = (int)_OUTPUT_MODE_.MODE_FROM_FIBER_IN;
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 1;
            }
            mxIrig.mxIrigbSetOutputSignalType(irigbCardHandle, _PORT_LIST_.PORT_1,
                (_SIGNAL_TYPE_)signalType, (_OUTPUT_MODE_)outputMode, inverse != 0);

            try
            {
                outputMode = (int)programName.GetValue("OutPort2OutputMode", 0);
                signalType = (int)programName.GetValue("OutPort2SignalType", 0);
                inverse = (int)programName.GetValue("OutPort2Inverse", 0);
            }
            catch
            {
                Console.WriteLine("OutPort2Output faile");
                outputMode = (int)_OUTPUT_MODE_.MODE_FROM_FIBER_IN;
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 1;
            }
            mxIrig.mxIrigbSetOutputSignalType(irigbCardHandle, _PORT_LIST_.PORT_2,
                (_SIGNAL_TYPE_)signalType, (_OUTPUT_MODE_)outputMode, inverse != 0);

            try
            {
                outputMode = (int)programName.GetValue("OutPort3OutputMode", 0);
                signalType = (int)programName.GetValue("OutPort3SignalType", 0);
                inverse = (int)programName.GetValue("OutPort3Inverse", 0);
            }
            catch
            {
                Console.WriteLine("OutPort3Output faile");
                outputMode = (int)_OUTPUT_MODE_.MODE_FROM_FIBER_IN;
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 1;
            }
            mxIrig.mxIrigbSetOutputSignalType(irigbCardHandle, _PORT_LIST_.PORT_3,
                (_SIGNAL_TYPE_)signalType, (_OUTPUT_MODE_)outputMode, inverse != 0);

            try
            {
                outputMode = (int)programName.GetValue("OutPort4OutputMode", 0);
                signalType = (int)programName.GetValue("OutPort4SignalType", 0);
                inverse = (int)programName.GetValue("OutPort4Inverse", 0);
            }
            catch
            {
                Console.WriteLine("OutPort4Output faile");
                outputMode = (int)_OUTPUT_MODE_.MODE_FROM_FIBER_IN;
                signalType = (int)_SIGNAL_TYPE_.TYPE_TTL;
                inverse = 1;
            }
            mxIrig.mxIrigbSetOutputSignalType(irigbCardHandle, _PORT_LIST_.PORT_4,
                (_SIGNAL_TYPE_)signalType, (_OUTPUT_MODE_)outputMode, inverse != 0);

            int ppsWidth = 0;
            try
            {
                ppsWidth = (int)programName.GetValue("PpsOutputWidth", 0);
            }
            catch { }
            mxIrig.mxIrigbSetPpsWidth(irigbCardHandle, ppsWidth);

            int outputParity = 0;
            try
            {
                outputParity = (int)programName.GetValue("IrigBOutputParity", 0);
            }
            catch { }
            mxIrig.mxIrigbSetOutputParityCheckMode(irigbCardHandle, (_PARITY_CHECK_MODE_)outputParity);

            //
            // Sync Time Source
            int timeSource = 0;
            try
            {
                timeSource = (int)programName.GetValue("SyncTimeSource", 0);
            }
            catch { }
            mxIrig.mxIrigbSetSyncTimeSrc(irigbCardHandle, (_RTC_SYNC_SOURCE_)timeSource);

            int syncTime = 0;
            int syncTimeEnable = 0;
            try
            {
                syncTime = (int)programName.GetValue("SyncTime", 0);
                syncTimeEnable = (int)programName.GetValue("SyncTimeEnable", 0);
            }
            catch { }

            programName.Close();
            programName = null;

            if (timerSyncPeriod != null)
            {
                timerSyncPeriod.Dispose();
                timerSyncPeriod = null;
            }

            if ((syncTimeEnable > 0) && (syncTime > 0))
            {
                timerSyncPeriod = new Timer(TimerCallback, null, 0, syncTime * 1000);
            }

        }

        private void TimerCallback(Object o)
        {
#if DEBUG
            Console.WriteLine("Sync. Time From IRIG RTC...");
#endif
            mxIrig.mxIrigbSyncTime(this.irigbCardHandle, false);
        }

        void ReInitSyncTime(object state, bool timedOut)
        {
#if DEBUG
            Console.WriteLine("ReInitSyncTime...");
#endif
            Init();
        }
    }
}
